/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx;

import ipdlx.tools.RandomGenerator;

import java.util.Enumeration;
import java.util.Vector;

/**
 * Base class for all games
 * 
 * @author Jan Humble
 */
public abstract class Game implements PDValues {

	protected long moveTimeLimit = 3000; // 3 secs

	/** how many rounds (iterations) */
	private int nrOfRounds;

	/** how many rounds (iterations) */
	protected float stDev = 0.0f;

	/** all players */
	protected Vector players;

	/** info about the result of this game */
	protected GameResult gameResult;

	/** payoff matrix of the game */
	protected GameMatrix gameMatrix;

	/**
	 * The percentage of mistakes the strategy makes. Mistake is a situation
	 * like this for example: strategy wants to play COOPERATE but for some
	 * unknown reasons it plays DEFECT.
	 */
	protected float probabilityOfMistake;

	protected Vector gameListeners;

	protected boolean broadcastResult = true;

	protected boolean broadcastRoundResults = false;

	/**
	 * Constructs a new game with the same game parameters as the given one.
	 * Also, all game listeners are transferred.
	 * 
	 * @param game
	 *            a template game to copy parameters from
	 * 
	 */
	public Game(Game game) {
		this(game.nrOfRounds, game.gameMatrix, game.probabilityOfMistake);
		this.stDev = game.getNrRoundsStDev();
		this.gameListeners = game.gameListeners;
	}

	/**
	 * constructor
	 * 
	 * @param nrOfRounds
	 *            number of iterations
	 * @param gameMatrix
	 *            matrix of the prisoner's dilemma game
	 * 
	 */
	public Game(int nrOfRounds, GameMatrix gameMatrix) {
		this(nrOfRounds, gameMatrix, 0.0f);
	}

	/**
	 * constructor
	 * 
	 * @param nrOfRounds
	 *            number of iterations
	 * @param gameMatrix
	 *            matrix of the prisoner's dilemma game
	 * @param noise
	 *            probability of mistake
	 */
	public Game(int nrOfRounds, GameMatrix gameMatrix, float noise) {
		this.nrOfRounds = nrOfRounds;
		this.gameMatrix = gameMatrix;
		this.probabilityOfMistake = noise;
	}

	/**
	 * Returns a small description of the current game parameters
	 */
	public String toString() {
		return ("Game (nrRounds= " + nrOfRounds + " stdev= " + stDev
				+ " noise= " + probabilityOfMistake + ")\n" + gameMatrix
				.toString());
	}

	/**
	 * Sets the number of rounds to play
	 * 
	 * @param nrOfRounds
	 *            number of iterations to play
	 */
	public final void setNumberOfRounds(int nrOfRounds) {
		this.nrOfRounds = nrOfRounds;
	}

	/**
	 * Returns a deviation of the current number of rounds and current standard
	 * deviation setting.
	 */
	public final int getNumberOfRoundsStDev() {
		int nrRounds = (int) RandomGenerator.gaussianRandom(nrOfRounds, stDev);
		if (nrRounds < 0) {
			nrRounds = 0;
		}
		return nrRounds;
	}

	/**
	 * Returns the current number of rounds.
	 * 
	 * @param useStDev
	 *            if true returns a deviation from the currently set number of
	 *            rounds to play.
	 */
	public final int getNumberOfRounds(boolean useStDev) {
		return useStDev ? getNumberOfRoundsStDev() : this.nrOfRounds;
	}

	/**
	 * Returns the currently set number of rounds.
	 */
	public final int getNumberOfRounds() {
		return getNumberOfRounds(false);
	}

	/**
	 * Returns the game matrix.
	 */
	public final GameMatrix getGameMatrix() {
		return this.gameMatrix;
	}

	/**
	 * Adds a player to this game instance.
	 * 
	 * @param player
	 *            the player to add.
	 */
	public void addPlayer(Player player) {
		if (players == null) {
			players = new Vector();
		}
		players.add(player);
	}

	/**
	 * Adds an array of players to this game instance.
	 * 
	 * @param players
	 *            the players to add.
	 */
	public void addPlayers(Vector players) {
		if (this.players == null) {
			this.players = new Vector();
		}
		players.addAll(players);
	}

	/**
	 * Sets the standard deviation for the number of rounds to play.
	 * 
	 * @param stDev
	 *            the standard deviation
	 */
	public final void setNrRoundsStDev(float stDev) {
		this.stDev = stDev;
	}

	/**
	 * Returns the standard deviation set for the number of rounds to play.
	 */
	public final float getNrRoundsStDev() {
		return stDev;
	}

	/**
	 * Sets the players to play this game.
	 * 
	 * @param players
	 *            the players
	 */
	public void setPlayers(Vector players) {
		this.players = players;
	}

	/**
	 * Sets the game matrix.
	 * 
	 * @param gameMatrix
	 *            the game matrix.
	 */
	public void setGameMatrix(GameMatrix gameMatrix) {
		this.gameMatrix = gameMatrix;
	}

	/**
	 * Sets the time limit for a player move at each iteration.
	 * 
	 * @param timeLimitMillis
	 *            the time limit in milliseconds.
	 */
	public void setMoveTimeLimit(long timeLimitMillis) {
		this.moveTimeLimit = timeLimitMillis;
	}

	/**
	 * Returns the move from a given strategy. This is by default used to check
	 * time limit considerations and log moves.
	 * 
	 * @param strategy
	 *            the strategy to acquire the move from.
	 */
	protected double getStrategyMove(Strategy strategy) {
		return getStrategyMove(strategy, true);
	}

	/**
	 * Returns the move from a given strategy. This is by default used to check
	 * time limit considerations and log moves.
	 * 
	 * @param strategy
	 *            the strategy to acquire the move from.
	 * @param checkTimeLimit
	 *            if true checks whether to check the play time limit
	 *            constraints imposed.
	 */
	protected double getStrategyMove(Strategy strategy, boolean checkTimeLimit) {
		long timeBefore = System.currentTimeMillis();
		double move = strategy.getMove();
		long timeAfter = System.currentTimeMillis();
		long total = timeAfter - timeBefore;
		// System.out.println(System.currentTimeMillis() + " Strategy '" +
		// strategy + "' taking " + total + " ms");

		if (checkTimeLimit && total > moveTimeLimit) {
			System.out.println("WARNING: Strategy '" + strategy
					+ "' overruning allowed move time (" + total + " ms > "
					+ moveTimeLimit + " ms !");
		}
		return move;
	}

	/**
	 * Returns a possibly modified move from another. Mainly used for
	 * introducing noise in the move signal.
	 * 
	 * @param move
	 *            the original move to filter.
	 */
	public double filterMove(double move) {

		if (Math.random() < probabilityOfMistake) {
			// mistake - change the move;
			return (move == COOPERATE) ? DEFECT : COOPERATE;
		}

		// there was no mistake - return the original move
		return move;
	}

	/**
	 * Returns the probability of mistake setting for all moves in the current
	 * game state.
	 * 
	 */
	public final float getProbabilityOfMistake() {
		return probabilityOfMistake;
	}

	/*
	 * Sets the probability of mistake for the current game.
	 * 
	 * @param probabilityOfMistake
	 */
	public final void setProbabilityOfMistake(float probabilityOfMistake) {
		this.probabilityOfMistake = probabilityOfMistake;
	}

	public final void removeGameListener(GameListener gameListener) {
		if (gameListeners != null) {
			gameListeners.remove(gameListener);
		}
	}

	public final Vector getGameListeners() {
		return this.gameListeners;
	}

	public void setBroadcastResult(boolean broadcast) {
		this.broadcastResult = broadcast;
	}

	public void setBroadcastRoundResults(boolean broadcast) {
		this.broadcastRoundResults = broadcast;
	}

	public boolean getBroadcastResult() {
		return this.broadcastResult;
	}

	public boolean getBroadcastRoundResults() {
		return this.broadcastRoundResults;
	}

	public final void addGameListener(GameListener gameListener) {
		if (gameListeners == null) {
			gameListeners = new Vector();
		}
		gameListeners.add(gameListener);
	}

	protected final void broadcastResults(GameResult result) {
		if (broadcastResult && gameListeners != null) {
			for (Enumeration e = gameListeners.elements(); e.hasMoreElements();) {
				((GameListener) e.nextElement()).gameResultPosted(result);
			}
		}
	}

	protected final void broadcastRoundResults(GameRoundResult result) {
		if (broadcastResult && gameListeners != null) {
			for (Enumeration e = gameListeners.elements(); e.hasMoreElements();) {
				((GameListener) e.nextElement()).gameRoundResultPosted(result);
			}
		}
	}

	protected final void broadcastGameStart(Game game) {
		if (gameListeners != null) {
			for (Enumeration e = gameListeners.elements(); e.hasMoreElements();) {
				((GameListener) e.nextElement()).gameStarted(game);
			}
		}
	}

	/**
	 * plays the game and return the results of it
	 * 
	 * @return result of the game
	 */
	public GameResult play() {
		// clears strategy "memory" -
		// important for strategies which takes actions
		// for example according to last opponent's move
		for (Enumeration e = players.elements(); e.hasMoreElements();) {
			((Player) e.nextElement()).clearMemory();
		}
		final int nrOfRounds = getNumberOfRounds(true);
		gameResult = new GameResult(players, nrOfRounds);
		double[] moves = new double[players.size()];
		for (int i = 0; i < nrOfRounds; i++) {

			// Calculate all moves
			int playerIndex = 0;
			for (Enumeration e = players.elements(); e.hasMoreElements(); playerIndex++) {
				moves[playerIndex] = filterMove(((Player) e.nextElement())
						.getMove());
			}

			// Calculate results
			final int nrOfPlayers = players.size();
			double[] payoffs = new double[players.size()];
			for (int currentIndex = 0; currentIndex < nrOfPlayers; currentIndex++) {
				Player current = (Player) players.elementAt(currentIndex);
				double[] opponentsMoves = new double[players.size() - 1];
				int opponentMoveIndex = 0;
				for (int opponentIndex = currentIndex + 1; opponentIndex < nrOfPlayers; opponentIndex++) {
					/*
					 * Check whether the same opponent. Note that a player might
					 * play against itself, hence we should not match Player
					 * instances.
					 */
					double currentMove = moves[currentIndex];
					double opponentMove = moves[opponentIndex];
					double[] payoff = gameMatrix.getPayoff(currentMove,
							opponentMove);
					payoffs[currentIndex] += payoff[0];
					payoffs[opponentIndex] += payoff[1];
					opponentsMoves[opponentMoveIndex++] = opponentMove;
				}

				// Current player has now played against all others
				current.setLastResult(payoffs[currentIndex]);
				current.setOpponentsMoves(opponentsMoves);
				gameResult.addPayoff(currentIndex, payoffs[currentIndex]);
				current.increaseResult(payoffs[currentIndex]);
			}
		}

		if (gameListeners != null) {
			broadcastResults(gameResult);
		}
		return gameResult;
	}
}
