/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */
package ipdlx;

/**
 * 
 * @author Jan Humble
 */
public class GameMatrix {

	protected double[][] payoffMatrix;

	public static final double[][] defaultPayoffMatrix5x5 = new double[][] {
			{ 4.0, 3.0, 2.0, 1.0, 0.0 }, { 4.25, 3.25, 2.25, 1.25, 0.25 },
			{ 4.5, 3.5, 2.5, 1.5, 0.5 }, { 4.75, 3.75, 2.75, 1.75, 0.75 },
			{ 5.0, 4.0, 3.0, 2.0, 1.0 } };

	public static final double[][] defaultPayoffMatrix2x2 = new double[][] {
			{ 3.0, 0.0 }, { 5.0, 1.0 } };

	protected int nrRows, nrColumns;

	public GameMatrix() throws WrongGameMatrixValuesException {
		this(defaultPayoffMatrix2x2);
	}

	public boolean checkValues(double[][] payoffMatrix) {
		return (payoffMatrix != null && payoffMatrix[0] != null && payoffMatrix.length == payoffMatrix[0].length);
	}

	public synchronized void setPayoffMatrix(double[][] payoffMatrix)
			throws WrongGameMatrixValuesException {
		if (checkValues(payoffMatrix)) {
			this.payoffMatrix = payoffMatrix;
			this.nrRows = payoffMatrix.length;
			this.nrColumns = payoffMatrix[0].length;
		} else {
			throw (new WrongGameMatrixValuesException(
					"Error: Wrong payoff matrix values."));
		}
	}

	public GameMatrix(double[][] payoffMatrix)
			throws WrongGameMatrixValuesException {
		setPayoffMatrix(payoffMatrix);
	}

	protected int[] getPayoffIndex(double moveA, double moveB) {
		int i = (int) Math.rint((nrRows - 1) - (moveA * (double) (nrRows - 1)));
		int j = (int) Math.rint((nrColumns - 1)
				- (moveB * (double) (nrColumns - 1)));
		return new int[] { i, j };
	}

	public double getPayoff(double moveA, double moveB, int player) {
		return getPayoff(moveA, moveB)[player];
	}

	public double[] getPayoff(double moveA, double moveB) {
		int[] index = getPayoffIndex(moveA, moveB);
		int i = index[0];
		int j = index[1];
		return new double[] { payoffMatrix[i][j], payoffMatrix[j][i] };
	}

	public double[][] getPayoffMatrix() {
		return payoffMatrix;
	}

	public String toString() {
		String out = "Payoff Matrix:\n";
		for (int j = 0; j < nrRows; j++) {
			for (int i = 0; i < nrColumns; i++) {
				out += payoffMatrix[j][i] + "\t";
			}
			out += "\n";
		}
		return out;
	}

	public static void main(String[] args) {
		try {
			GameMatrix gameMatrix = new GameMatrix();
			System.out.println(gameMatrix.toString());
			double[] payoff = gameMatrix.getPayoff(Double.parseDouble(args[0]),
					Double.parseDouble(args[1]));
			System.out.println("Payoff A=" + payoff[0] + " Payoff B="
					+ payoff[1]);
		} catch (WrongGameMatrixValuesException wgmv) {
		}

	}

}