/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx;

import java.text.MessageFormat;
import java.util.Vector;

/**
 * Results of the game how many points each player won
 * 
 * @author Jan Humble
 */
public class GameResult {
	/** number of points won by players */
	protected final double[] payoffs;

	protected final Player[] players;

	protected final int nrRounds;

	/** game log - what moves were chosen in every round */
	private double[][] log;

	/**
	 * constructor
	 */
	public GameResult(int nrRounds) {
		this.payoffs = null;
		this.players = null;
		this.nrRounds = nrRounds;
	}

	/**
	 * constructor
	 */
	public GameResult(Vector players, int nrRounds) {
		this((Player[]) players.toArray(new Player[players.size()]), nrRounds);
	}

	/**
	 * constructor
	 */
	public GameResult(Player[] players, int nrRounds) {
		payoffs = new double[players.length];
		this.players = players;
		this.nrRounds = nrRounds;
		this.log = new double[players.length][nrRounds];
	}

	public final Player getPlayer(int index) {
		return this.players[index];
	}

	public void addPayoff(int index, double payoff) {
		payoffs[index] += payoff;
	}

	/**
	 * returns number of points won by each player during this game
	 * 
	 * @return payoff number of points won by each player
	 */
	public double[] getPayoffs() {
		return payoffs;
	}

	/**
	 * returns number of points won by selected player during this game
	 * 
	 * @param playerNb
	 *            player A or B
	 * @return payoff number of points won selected player
	 */
	public final double getPayoffPlayer(int playerNb) {
		return this.payoffs[playerNb];
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer buff = new StringBuffer("*** Game Results (nrRounds = "
				+ nrRounds + "):");
		// \nPlayer\tStrategy\tPoints");
		for (int i = 0; i < players.length; i++) {
			Object[] args = { players[i].getName(),
					players[i].getStrategy().getFullName(),
					new Double(getPayoffPlayer(i)) };
			buff.append(MessageFormat.format(
					"\n player={0} strategy={1} points={2}", args));
			// buff.append(MessageFormat.format("\n{0}\t{1}\t{2}", args));
		}
		return buff.toString();
	}

	/**
	 * write type of move made by players to game log
	 * 
	 * @param nbOfRound -
	 *            which round is this
	 * @param player -
	 *            which player A or B
	 * @param move -
	 *            type of move of player - COOPERATION or DEFECTION
	 */
	public void writeMove(int nbOfRound, int player, double move) {
		log[player][nbOfRound] = move;
	}

	/**
	 * returns game log (for both players)
	 * 
	 * @return game log
	 */
	public double[][] getLog() {
		return log;
	}

}
