/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx;

/**
 * Standard 2 player game
 * 
 * @author Tomasz Kaczanowski, Jan Humble
 */
public class StandardGame extends Game {

	/** first player */
	protected Player playerA;

	/** second player */
	protected Player playerB;

	/**
	 * Constructs a new game with the same game parameters as the given one.
	 * Also, all game listeners are transferred.
	 * 
	 * @param game
	 *            a template game to copy parameters from
	 * 
	 */
	public StandardGame(Game game) {
		super(game);
	}

	/**
	 * Constructs a standard game, playing 200 rounds with the default game
	 * matrix.
	 */
	public StandardGame() {
		this(200, new GameMatrix());
	}

	/**
	 * constructor
	 * 
	 * @param nrOfRounds
	 *            number of iterations
	 * @param gameMatrix
	 *            matrix of the prisoner's dilemma game
	 */
	public StandardGame(int nrOfRounds, GameMatrix gameMatrix) {
		this(nrOfRounds, gameMatrix, 0.0f);
	}

	/**
	 * constructor
	 * 
	 * @param nrOfRounds
	 *            number of iterations
	 * @param gameMatrix
	 *            matrix of the prisoner's dilemma game
	 * @param noise
	 *            probability of mistake
	 */
	public StandardGame(int nrOfRounds, GameMatrix gameMatrix, float noise) {
		super(nrOfRounds, gameMatrix, noise);
	}

	/**
	 * sets the first player
	 * 
	 * @param playerA
	 *            first player of this game
	 */
	public final void setPlayerA(Player playerA) {
		this.playerA = playerA;
		addPlayer(playerA);
	}

	public final Player getPlayerA() {
		return playerA;
	}

	public final Player getPlayerB() {
		return playerB;
	}

	/**
	 * sets the second player
	 * 
	 * @param playerB
	 *            second player of this game
	 */
	public void setPlayerB(Player playerB) {
		this.playerB = playerB;
		addPlayer(playerB);
	}

	/**
	 * plays the game and return the results of it
	 * 
	 * @return result of the game
	 */
	public GameResult play() {
		// clears strategy "memory" - important for strategies which takes
		// actions
		// for example according to last opponent's move
		playerA.clearMemory();
		playerB.clearMemory();

		final int nrOfRounds = getNumberOfRounds(true);
		StandardGameResult gameResult = new StandardGameResult(playerA,
				playerB, nrOfRounds);

		this.gameResult = gameResult;

		Strategy strategyA = playerA.getStrategy(true);
		Strategy strategyB = playerB.getStrategy(true);

		if (strategyA == strategyB) {
			System.out.println("Creating strategy clone for "
					+ strategyB.getName());
			try {
				strategyB = (Strategy) strategyA.clone();
			} catch (CloneNotSupportedException cnse) {
				cnse.printStackTrace();
				return gameResult;
			}
		}

		broadcastGameStart(this);

		for (int i = 0; i < nrOfRounds; i++) {

			double moveA = filterMove(getStrategyMove(strategyA));
			double moveB = filterMove(getStrategyMove(strategyB));
			double lastResultA = gameMatrix.getPayoff(moveA, moveB, PLAYER_A);
			double lastResultB = gameMatrix.getPayoff(moveA, moveB, PLAYER_B);
			gameResult.addPayoff(lastResultA, lastResultB);
			gameResult.writeMove(i, PLAYER_A, moveA);
			gameResult.writeMove(i, PLAYER_B, moveB);
			strategyA.setOpponentMove(moveB);
			strategyB.setOpponentMove(moveA);
			strategyA.setLastResult(lastResultA);
			strategyB.setLastResult(lastResultB);
			if (broadcastRoundResults) {
				broadcastRoundResults(new GameRoundResult(i + 1, strategyA,
						moveA, lastResultA, strategyB, moveB, lastResultB));
			}
		}

		// the game is over, so we can increase players' results
		double resultA = gameResult.getPayoffPlayer(PLAYER_A);
		double resultB = gameResult.getPayoffPlayer(PLAYER_B);

		playerA.gamesPlayed++;
		playerB.gamesPlayed++;

		playerA.increaseResult(resultA);
		playerB.increaseResult(resultB);

		if (resultA > resultB) {
			playerA.gamesWon++;
			playerB.gamesLost++;
		} else if (resultA < resultB) {
			playerA.gamesLost++;
			playerB.gamesWon++;
		} else if (resultA == resultB) {
			playerA.gamesTied++;
			playerB.gamesTied++;
		}

		if (gameListeners != null) {
			broadcastResults(gameResult);
		}
		return gameResult;
	}
}
