/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx;

import java.lang.reflect.InvocationTargetException;

/**
 * Base abstract class for all strategies.
 * 
 * @author Tomasz Kaczanowski, Jan Humble
 */
public abstract class Strategy implements PDValues, Cloneable {
	/**
	 * strategy abbreviated name - eg. ALLC
	 * 
	 * @todo
	 */
	private String abbrName;

	/** strategy full name - eg. Always Cooperate */
	private String name;

	/** strategy description - how does this particular strategy works */
	private String description;

	/**
	 * info about last opponent's move - did he defect or cooperate. *not every
	 * strategy uses this variable
	 */
	protected double opponentMove;

	/**
	 * info about last opponent's move - did he defect or cooperate. *not every
	 * strategy uses this variable
	 */
	protected double[] opponentsMoves;

	/**
	 * info about last game's result. not every strategy uses this variable
	 */
	protected double lastResult;

	/**
	 * sum of points gathered by this strategy. useful if many players use one
	 * strategy and you want to know how many points did the strategy win in
	 * total
	 */
	private double result;

	protected int nrOfOpponents = 1;

	/**
	 * Constructor for "dirty" strategies. Sets: result to 0 also sets strategy
	 * names and description
	 * 
	 * @param abbrName
	 *            abbreviated name of strategy, eg. "ALLC"
	 * @param name
	 *            name of strategy, eg. "Always Cooperate"
	 * @param description
	 *            description of strategy, how does it works, when does it
	 *            coopearate and defect
	 */
	public Strategy(String abbrName, String name, String description) {
		this.abbrName = abbrName;
		this.name = name;
		this.description = description;
		reset();
	}

	/**
	 * returns move
	 * 
	 * @return move of this strategy
	 */
	public abstract double getMove();

	/**
	 * adds i point to result
	 * 
	 * @param i
	 *            number of points to add to result
	 */
	final void increaseResult(double i) {
		result += i;
	}

	public void setLastResult(double lastResult) {
		this.lastResult = lastResult;
	}

	public final int getNumberOfOpponents() {
		return this.nrOfOpponents;
	}

	public final void setNumberOfOpponents(int nrOfOpponents) {
		this.nrOfOpponents = nrOfOpponents;
	}

	/**
	 * returns strategy result
	 * 
	 * @return strategy's result (the number of points gathered during game(s)
	 */
	protected final double getResult() {
		return result;
	}

	/**
	 * returns strategy description
	 * 
	 * @return strategy's description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * returns strategy name
	 * 
	 * @return strategy's name
	 */
	public String getName() {
		return name;
	}

	/**
	 * returns strategy abbreviated name
	 * 
	 * @return strategy's abbreviated name
	 */
	public String getAbbrName() {
		return abbrName;
	}

	/**
	 * returns full name of the strategy
	 * 
	 * @return full name of the strategy "abbrName (FullName)"
	 */

	protected Object clone() throws CloneNotSupportedException {
		try {
			// Strategy newStrategy = (Strategy) getClass().newInstance();
			Strategy newStrategy = (Strategy) (getClass()
					.getConstructor(new Class[] {})).newInstance((Object[])null);
			return newStrategy;
		} catch (NoSuchMethodException nsme) {
			nsme.printStackTrace();
			return null;
		} catch (InvocationTargetException ite) {
			ite.printStackTrace();
			return null;
		} catch (InstantiationException ie) {
			ie.printStackTrace();
			return null;
		} catch (IllegalAccessException iae) {
			iae.printStackTrace();
			return null;
		}
	}

	public String getFullName() {
		StringBuffer buf = new StringBuffer(abbrName);
		buf.append(" (");
		buf.append(name);
		buf.append(")");

		return buf.toString();
	}

	/**
	 * setter for opponentMove field. if your strategy doesn't use last opponent
	 * move in its next move calculations, setting of this field won't hurt it
	 * 
	 * @param opponentMove
	 *            last move of the opponent
	 */
	public final void setOpponentMove(double opponentMove) {
		this.opponentMove = opponentMove;
		setOpponentsMoves(new double[] { opponentMove });
	}

	/**
	 * setter for opponentsMoves field. if your strategy doesn't use last
	 * opponent move in its next move calculations, setting of this field won't
	 * hurt it
	 * 
	 * @param opponentsMoves
	 *            last move of all the opponents
	 */
	public final void setOpponentsMoves(double[] opponentsMoves) {
		this.opponentsMoves = opponentsMoves;
	}

	public final double[] getOpponentsMoves() {
		return this.opponentsMoves;
	}

	public final double getOpponentMove() {
		return this.opponentMove;
	}

	/**
	 * @deprecated This used to implement noise, but is now implemeneted in the
	 *             Game class.
	 */
	public final double getFinalMove(double move) {
		return move;
	}

	/**
	 * sets the initial state of this strategy
	 */
	public void reset() {
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append(getFullName());
		return buf.toString();
	}
}
