/*
 <COPYRIGHT>

 Copyright (c) 2004-2006, University of Nottingham
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 - Neither the name of the ...organisation/regents...
 nor the names of its contributors may be used to endorse or promote products
 derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 </COPYRIGHT>

 Created by: Jan Humble (University of Nottingham)
 
 */

package ipdlx.gui;

import ipdlx.Game;
import ipdlx.GameListener;
import ipdlx.GameResult;
import ipdlx.GameRoundResult;
import ipdlx.StandardGame;
import ipdlx.Tournament;
import ipdlx.TournamentListener;
import ipdlx.TournamentResult;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.filechooser.FileFilter;

/**
 * 
 * @author Jan Humble
 */
public class InfoPanel extends JPanel implements GameListener,
		TournamentListener, ActionListener {

	private JTextArea textArea;

	private Tournament tournament;

	private static final String INTRO = "WELCOME TO IPDLX TOURNAMENT\n"
			+ "Written by Jan Humble <jan.humble@nottingham.ac.uk>";

	private File chosenDirectory;

	protected boolean autoClear = true;

	private JCheckBox autoClearCB;

	private boolean fileOutput = false;

	private File outputFile = null;

	private FileWriter fileWriter = null;

	public InfoPanel(Tournament tournament) {
		super(new BorderLayout());
		this.tournament = tournament;
		this.textArea = new JTextArea(INTRO);
		// this.textArea.setTabSize(20);
		this.textArea.setEditable(false);
		add(new JScrollPane(textArea));
		JPanel p = new JPanel();

		autoClearCB = new JCheckBox("Auto clear results", autoClear);
		autoClearCB.addActionListener(this);
		p.add(autoClearCB);

		JButton clearB = new JButton("Clear");
		clearB.addActionListener(this);
		p.add(clearB);

		JButton infoB = new JButton("Print Game Info");
		infoB.addActionListener(this);
		p.add(infoB);

		JButton b = new JButton("Save to file ...");
		b.addActionListener(this);
		p.add(b);

		JCheckBox fileOutputCB = new JCheckBox("file output");
		fileOutputCB.addActionListener(this);
		fileOutputCB.setSelected(fileOutput);
		p.add(fileOutputCB);

		add(BorderLayout.SOUTH, p);
		tournament.getGame().addGameListener(this);
		tournament.addTournamentListener(this);
	}

	public void appendText(String text) {
		if (fileOutput) {
			try {
				fileWriter.write(text);

			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			textArea.append(text);
		}
	}

	public void setFileOutput(boolean fileOutput) {
		this.fileOutput = fileOutput;
		if (fileOutput) {
			outputFile = new File(createTimeTaggedName());
			try {
				fileWriter = new FileWriter(outputFile, true);
				textArea.append("All output diverted to file: "
						+ outputFile.getAbsolutePath());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		} else {
			closeFileOutput();
		}
	}

	protected void closeFileOutput() {
		if (fileWriter != null) {
			try {
				// fileWriter.flush();
				fileWriter.close();
				fileWriter = null;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (outputFile != null) {
			outputFile = null;
		}
	}

	public void setText(String text) {
		textArea.setText(text);
	}

	public void tournamentResultPosted(TournamentResult result) {
		appendText("\n" + result.toString());
		if (autoClear) {
			if (fileOutput) {
				closeFileOutput();
			}
		}
	}

	public void gameResultPosted(GameResult result) {
		appendText("\n" + result.toString());
	}

	public void gameRoundResultPosted(GameRoundResult result) {
		appendText("\n" + result.toCompactString());
	}

	public void gameStarted(Game game) {
		if (game instanceof StandardGame) {
			appendText("\n*** Game started: "
					+ ((StandardGame) game).getPlayerA().getStrategy() + " vs "
					+ ((StandardGame) game).getPlayerB().getStrategy());
		}
	}

	public void tournamentStarted(Tournament t) {
		if (autoClear) {
			textArea.setText(INTRO + "\n");
			if (fileOutput) {
				closeFileOutput();
				setFileOutput(true);
			}
		}
		appendText("\nTournament Started ...");
	}

	private File chooseSaveFile(String defaultName) {
		if (chosenDirectory == null) {
			chosenDirectory = new File(".");
		}
		JFileChooser chooser = new JFileChooser(chosenDirectory);
		TextFileFilter filter = new TextFileFilter();
		chooser.setFileFilter(filter);
		chooser.setSelectedFile(new File(defaultName));
		int returnVal = chooser.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			System.out.println("You chose to save to this file: "
					+ chooser.getSelectedFile().getName());
			chosenDirectory = chooser.getCurrentDirectory();
			return chooser.getSelectedFile();
		}
		return null;
	}

	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		if (command.equals("Clear")) {
			textArea.setText(INTRO + "\n");
		} else if (command.equals("Print Game Info")) {
			textArea.append(tournament.toString());
		} else if (command.equals("Save to file ...")) {

			File file = chooseSaveFile(createTimeTaggedName());
			try {
				String text = textArea.getText();
				FileWriter fw = new FileWriter(file);
				fw.write(text, 0, text.length());
				fw.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		} else if (command.equals("Auto clear results")) {
			autoClear = autoClearCB.isSelected();
		} else if (command.equals("file output")) {
			setFileOutput(((JCheckBox) ae.getSource()).isSelected());
		}
	}

	static String createTimeTaggedName() {
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd_HHmmss");
		String date = dateFormat.format(java.util.Calendar.getInstance()
				.getTime());
		return new String("pd_output_" + date + ".txt");
	}

}

class TextFileFilter extends FileFilter {

	public boolean accept(File f) {
		if (!f.canRead()) {
			return false;
		}

		if (f.isFile()) {
			String filename = f.getName();
			if (filename.endsWith(".txt")) {
				return true;
			}
		} else if (f.isDirectory()) {
			return true;
		}
		return false;
	}

	public String getDescription() {
		return "Text Output File";
	}
}
