/*
 * IPDL - Iterated Prisoner's Dilemma Library
 * Copyright (C) 2003-2004 by Tomasz Kaczanowski
 * giaur @ qs dot pl
 * http://www.giaur.qs.pl/ipdl
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ipdlx.strategy;

import ipdlx.Strategy;
import java.util.*;
import java.lang.Math;

/**
 * CMC class sample strategy
 * 
 * @author Andrew Fiore
 */
public class CMCSample extends Strategy {
	private final static String _abbrName = "CMCS";

	private final static String _name = "CMC Sample Strategy";

	private final static String _description = "Not a proper strategy -- just a bit of code to show how to do different things with the Strategy class.";

	private ArrayList myPreviousMoves;
	private ArrayList opponentPreviousMoves;
	int moveIndex = 0;

	private double previousOpponentMove;
	
	private int historyLength;
	private int firstMove;
	
	private int getHistoryLength() {
		return historyLength;
	}
	
	private void resetHistory() {
		historyLength = 0;
		firstMove = 1;
		myPreviousMoves = new ArrayList();
		opponentPreviousMoves = new ArrayList();
	}

	public CMCSample() {
		super(_abbrName, _name, _description);
		resetHistory();
	}

	protected CMCSample(String abbrName, String name, String description) {
		super(abbrName, name, description);
		resetHistory();
	}
	
	private void updateMyHistory(double myMove) {
//		System.out.println("Adding my move " + myMove);
		myPreviousMoves.add(new Double(myMove));
		historyLength++;
	}

	private void updateOpponentHistory(double opponentMove) {
//		System.out.println("Adding opp move " + opponentMove);
		opponentPreviousMoves.add(new Double(opponentMove));
	}
	
	private ArrayList getMyPreviousMoves() {
		return myPreviousMoves;
	}
	
	private ArrayList getOpponentPreviousMoves() {
		return opponentPreviousMoves;
	}
	
	private List getMyPreviousNMoves(int n) {
		int fromIndex = 0;
		if (myPreviousMoves.size() > n) {
			fromIndex = myPreviousMoves.size() - n;
		}
		int toIndex = myPreviousMoves.size();
		return ((ArrayList)myPreviousMoves).subList(fromIndex, toIndex);
	}

	private List getOpponentPreviousNMoves(int n) {
		int fromIndex = 0;
		if (opponentPreviousMoves.size() > n) {
			fromIndex = opponentPreviousMoves.size() - n;
		}
		int toIndex = opponentPreviousMoves.size();
		return ((ArrayList)opponentPreviousMoves).subList(fromIndex, toIndex);
	}
	

	public void reset() {
		resetHistory();
	}
	
	public double getMove() {
		if (firstMove > 0) {
			firstMove = 0;
		} else {
			updateOpponentHistory(opponentMove);
		}

		double move = getMoveDecision();
		
		updateMyHistory(move);
		return move;
	}
	
	public double getMoveDecision() {
	
		/* 
		   This function should return a move for your strategy.  The constants
		   COOPERATE=1.0 and DEFECT=0.0 are defined, so you can just do, e.g.,
		     return COOPERATE;
		   if you would like.  However, you can also return a double value 
		   BETWEEN 0 and 1.  Values > 0.5 will be taken as COOPERATE, and
		   values < 0.5 will be taken as DEFECT.  This way, you don't have to
		   worry about rounding if you have some sort of probabilistic strategy.
		*/
	
	
		/*
		   Set a default move.  This way you'll be sure you return some valid
		   value.  It's also important because, on the first move, you'll have
		   no history for yourself or your opponent, so you can't perform any
		   operations on past moves to make a decision.
		*/
		double move = COOPERATE;

		
		/*
		   These two lines pick a move randomly with different probabilities.
		   Maybe you could average a random value and a deliberately chosen
		   value, too...?  e.g., move = (move + Math.random()) / 2
		   
		   Remeber that move values > 0.5 become COOPERATE and
		   values < 0.5 become DEFECT.  This is automatic; you don't need
		   to round them yourself -- just return the fractional values.
		*/
		
		// Randomize the move, 50% chance of cooperation
		// move = (Math.random() > 0.5) ? COOPERATE : DEFECT;
	
		/*  OR  */
		
		// Randomize the move, 75% chance of cooperation
		// move = (Math.random() > 0.25) ? COOPERATE : DEFECT;
		
		

		/*
		   Say you want to do something with recent moves you and your 
		   opponent have made.  We need to fetch the history for each strategy
		   and then loop over it.  This sample code fetches the most recent
		   historyWindowSize moves (set to 10 in this sample).
		   
		   History is reset between every game, so you won't have a record of
		   past matches with different strategies available.
        */
        
        /* // BEGIN LOOP SAMPLE CODE
        
		int historyWindowSize = 10;
		List myLastMoves = getMyPreviousNMoves(historyWindowSize);
		List opponentLastMoves = getOpponentPreviousNMoves(historyWindowSize);

		//
		// Say you want to average your opponent's moves, or sum them, or
		// compare them with your move on each turn, etc. ... you can use
		// this loop.  But you don't necessarily need to do this to write
		// a good PD strategy... just one possibility.
		//
		// In doing math on these moves, remember that COOPERATE=1.0 and 
		// DEFECT=0.0.
		//
		// To loop over the history, we need to set the number of loop
		// iterations.  Early in the game, historyWindowSize might be bigger
		// than the available history (i.e., getHistoryLength()), so we have
		// to use the minimum of the two as the upper bound for i in the loop.
		//
		
		int loopIterations = Math.min(historyWindowSize, getHistoryLength());

		for (int i = 0; i < loopIterations; i++) {
			double myMove = ((Double)myLastMoves.get(i)).doubleValue();
			double oppMove = ((Double)opponentLastMoves.get(i)).doubleValue();
			
			// Do something here...?
		}
		
		*/ // END LOOP SAMPLE CODE
		
		
		// Want to do something only if a certain number of turns have
		// already been taken?  Use getHistoryLength().  In this example,
		// we always defect after two turns into the game...
		/*
		if (getHistoryLength() > 2) {
			move = DEFECT;
		}
		*/

		// Don't forget to return some value for your move!
		return move;
	}
}
